##########################################################################
#
#  Maple TFL
#
#  filename: Language.mpl 
#  description: Constructors and utilities for Language type.
#	 
#  [to be included in tfl.mpl]
#	 
##########################################################################

define(  'Language',

                'accept',
                'reset',
                'getNextString',
                'getFirstStrings',
                'getLengthCut',

                'getAcceptor',   'setAcceptor',
                'getGenerator',  'setGenerator',
                'getExpression', 'setExpression',

                'whatChomskyType',
                'isEmpty',
                'isFinite'
        );

##########################################################################

read "Acceptor.mpl";
read "Generator.mpl";
read "Expression.mpl";

##########################################################################


# simple language constructor
newLanguage := proc()
  local x, l;
  if nargs = 0 then
    # create empty language
    printf( "lang: creating empty language\n");
    return module()
      export 
                accept,
                reset,
                getNextString,
                getFirstStrings,
                getLengthCut,

                getAcceptor,   setAcceptor,
                getGenerator,  setGenerator,
                getExpression, setExpression,

                whatChomskyType,
                isEmpty,
                isFinite
            ;
      local m_acceptor, m_generator, m_expression;
      accept := ( p_str ) -> FAIL;
      getAcceptor := () -> m_acceptor;
      setAcceptor := proc( p_acc )
            m_acceptor := eval(p_acc);
            # connect corresponding procedure to acceptor
            accept := (str) -> m_acceptor:-accept(str);
            end proc;
      getGenerator := () -> m_generator;
      setGenerator := proc( p_gen )
            m_generator := eval(p_gen);
            # connect corresponding procedures to generator
            reset := () -> m_generator:-reset();
            getNextString := () -> m_generator:-getNextString();
            end proc;
      getExpression := () -> m_expression;
      setExpression := proc( p_expr )
            m_expression := eval( p_expr );
            # connect corresponding procedures to expression
            # (nothing to connect yet)
            end proc;
    end module;

  else # nargs >= 1, we have something to construct language from
    x := args[1];
    if type( x, 'Acceptor' ) then
      printf( "newLanguage: got acceptor\n" );
      l := newLanguage();
      l :- setAcceptor( x );
      return l;
    elif type( x, 'Generator' ) then
      printf( "newLanguage: got generator\n" );
      l := newLanguage();
      l :- setGenerator( x );
      return l;
    elif type( x, 'Expression' ) then
      printf( "newLanguage: got expression\n" );
      l := newLanguage();
      l :- setExpression( x );
      return l;
    end if;
  end if;
  error "wrong arguments, couldn't create language";
end proc;
        


